@@ -20,10 +20,11 @@
 
 (* This file originally come from typing/oprint.ml *)
 
-open Format;
 open Outcometree;
 open Camlp4;
 
+value fprintf = Format_doc.fprintf;
+
 exception Ellipsis;
 value cautious f ppf arg =
   try f ppf arg with [ Ellipsis -> fprintf ppf "..." ]
@@ -85,8 +86,10 @@ value print_out_value ppf tree =
         end
     | Oval_list tl ->
         fprintf ppf "@[<1>[%a]@]" (print_tree_list print_tree ";") tl
-    | Oval_array tl ->
+    | Oval_array (tl, Asttypes.Mutable) ->
         fprintf ppf "@[<2>[|%a|]@]" (print_tree_list print_tree ";") tl
+    | Oval_array (_, Asttypes.Immutable) ->
+        assert False
     | Oval_constr (Oide_ident { printed_name = "true" }) [] -> fprintf ppf "True"
     | Oval_constr (Oide_ident { printed_name = "false" }) [] -> fprintf ppf "False"
     | Oval_constr name [] -> print_ident ppf name
@@ -95,7 +98,12 @@ value print_out_value ppf tree =
     | Oval_record fel ->
         fprintf ppf "@[<1>{%a}@]" (cautious (print_fields True)) fel
     | Oval_tuple tree_list ->
-        fprintf ppf "@[(%a)@]" (print_tree_list print_tree ",") tree_list
+        let tuple (lbl, x) =
+          match lbl with
+          [ Some _ -> assert False
+          | None -> x ]
+        in
+        fprintf ppf "@[(%a)@]" (print_tree_list print_tree ",") (List.map tuple tree_list)
     | Oval_ellipsis -> raise Ellipsis
     | Oval_printer f -> f ppf
     | tree -> fprintf ppf "@[<1>(%a)@]" (cautious print_tree) tree ]
@@ -175,7 +183,12 @@ and print_simple_out_type ppf =
   [ Otyp_var ng s -> fprintf ppf "'%s%s" (if ng then "_" else "") s
   | Otyp_constr id [] -> fprintf ppf "@[%a@]" print_ident id
   | Otyp_tuple tyl ->
-      fprintf ppf "@[<1>(%a)@]" (print_typlist print_out_type " *") tyl
+      let tuple (lbl, x) =
+        match lbl with
+        [ Some _ -> assert False
+        | None -> x ]
+      in
+      fprintf ppf "@[<1>(%a)@]" (print_typlist print_out_type " *") (List.map tuple tyl)
   | Otyp_stuff s -> fprintf ppf "%s" s
   | Otyp_variant (row_fields, closed, tags) ->
       let print_present ppf =
@@ -210,7 +223,7 @@ and print_simple_out_type ppf =
       fprintf ppf "@[<hv 2>{ %a }@]"
         (print_list print_out_label (fun ppf -> fprintf ppf ";@ ")) lbls
   | Otyp_abstract -> fprintf ppf "<abstract>"
-  | Otyp_module (p, fl) ->
+  | Otyp_module {opack_path=p; opack_cstrs=fl} ->
       do {
           fprintf ppf "@[<1>(module %a" print_ident p;
           let first = ref True in
@@ -243,7 +256,12 @@ and print_out_constr ppf constr =
   | (_,None) ->
       fprintf ppf "@[<2>%s of@ %a@]" name
         (print_typlist print_out_type " and") tyl ]
-and print_out_label ppf (name, mut, arg) =
+and print_out_label ppf out_label =
+  let { olab_name = name; olab_mut = mut; olab_type = arg } = out_label in
+  let mut = match mut with
+    [ Asttypes.Mutable -> True
+    | Asttypes.Immutable -> False ]
+  in
   fprintf ppf "@[<2>%s :@ %s%a@]" name (if mut then "mutable " else "")
     print_out_type arg
 and print_out_extension_constructor ppf ext =
@@ -318,7 +336,7 @@ value type_parameter ppf (ty, (var, inj)) =
   let open Asttypes in
   fprintf ppf "%s%s%s%s"
     (match inj with [ Injective -> "!" | NoInjectivity -> "" ])
-    (match var with [ Covariant -> "+" | Contravariant -> "-" | NoVariance -> "" ])
+    (match var with [ Covariant -> "+" | Contravariant -> "-" | Bivariant -> "+-" | NoVariance -> "" ])
     (if ty = "_" then "" else "'")
     ty
 ;
@@ -502,6 +520,10 @@ and print_out_type_decl kwd ppf { otype_name    = name
 
 (* Phrases *)
 
+value deprecated_printer f ppf x =
+  Format_doc.deprecated_printer (fun ppf -> f ppf x) ppf
+;
+
 value print_out_exception ppf exn outv =
   match exn with
   [ Sys.Break -> fprintf ppf "Interrupted.@."
@@ -509,7 +531,7 @@ value print_out_exception ppf exn outv =
   | Stack_overflow ->
       fprintf ppf "Stack overflow during evaluation (looping recursion?).@."
   | _ ->
-      fprintf ppf "@[Exception:@ %a.@]@." Toploop.print_out_value.val outv ]
+      fprintf ppf "@[Exception:@ %a.@]@." (deprecated_printer Toploop.print_out_value.val) outv ]
 ;
 
 value rec print_items ppf =
@@ -520,7 +542,7 @@ value rec print_items ppf =
         match valopt with
         [ Some v ->
             fprintf ppf "@[<2>%a =@ %a@]" Toploop.print_out_sig_item.val tree
-              Toploop.print_out_value.val v
+              (deprecated_printer Toploop.print_out_value.val) v
         | None -> fprintf ppf "@[%a@]" Toploop.print_out_sig_item.val tree ];
         if items <> [] then fprintf ppf "@ %a" print_items items else ()
       } ]
@@ -530,16 +552,16 @@ value print_out_phrase ppf =
   fun
   [ Ophr_eval outv ty ->
       fprintf ppf "@[- : %a@ =@ %a@]@." Toploop.print_out_type.val ty
-        Toploop.print_out_value.val outv
+        (deprecated_printer Toploop.print_out_value.val) outv
   | Ophr_signature [] -> ()
   | Ophr_signature items -> fprintf ppf "@[<v>%a@]@." print_items items
   | Ophr_exception (exn, outv) -> print_out_exception ppf exn outv ]
 ;
 
-Toploop.print_out_value.val := print_out_value;
+Toploop.print_out_value.val := Format_doc.compat print_out_value;
 Toploop.print_out_type.val := print_out_type;
 Toploop.print_out_class_type.val := print_out_class_type;
 Toploop.print_out_module_type.val := print_out_module_type;
 Toploop.print_out_sig_item.val := print_out_sig_item;
 Toploop.print_out_signature.val := print_out_signature;
-Toploop.print_out_phrase.val := print_out_phrase;
+Toploop.print_out_phrase.val := Format_doc.compat print_out_phrase;
